@@ -25,8 +25,7 @@
  * @{
  */
 
-#ifndef _GETLINE_H_
-#define _GETLINE_H_
+#pragma once
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -46,6 +45,4 @@
  */
 ssize_t knot_getline(char **lineptr, size_t *n, FILE *stream);
 
-#endif // _GETLINE_H_
-
 /*! @} */
