@@ -24,11 +24,10 @@
  * @{
  */
 
-#ifndef _KNOTD_REMOTE_H_
-#define _KNOTD_REMOTE_H_
+#pragma once
 
 #include "knot/conf/conf.h"
-#include "libknot/packet/packet.h"
+#include "libknot/packet/pkt.h"
 #include "libknot/rrset.h"
 #include "libknot/dnssec/key.h"
 #include "knot/server/server.h"
@@ -38,6 +37,7 @@
 
 /*!
  * \brief Bind RC interface according to configuration.
+ *
  * \param desc Interface descriptor (address, port).
  *
  * \retval socket if passed.
@@ -50,20 +50,22 @@ int remote_bind(conf_iface_t *desc);
  *
  * \note Breaks all pending connections.
  *
- * \param r RC interface socket
+ * \param desc Interface descriptor (address, port).
+ * \param socket Interface socket
  *
  * \retval KNOT_EOK on success.
  * \retval knot_error else.
  */
-int remote_unbind(int r);
+int remote_unbind(conf_iface_t *desc, int sock);
 
 /*!
  * \brief Poll new events on RC socket.
+ *
  * \param r RC interface socket.
  *
  * \return number of polled events or -1 on error.
  */
-int remote_poll(int r);
+int remote_poll(int sock);
 
 /*!
  * \brief Start a RC connection with remote.
@@ -76,19 +78,18 @@ int remote_poll(int r);
  * \return client TCP socket if success.
  * \return KNOT_ECONNREFUSED if fails to receive command.
  */
-int remote_recv(int r, sockaddr_t *a, uint8_t* buf, size_t *buflen);
+int remote_recv(int sock, struct sockaddr_storage *addr, uint8_t *buf,
+                size_t *buflen);
 
 /*!
  * \brief Parse a RC command.
  *
- * \param pkt Dst structure for parsed command.
- * \param buf Remote command in wire format.
- * \param buflen Wire format length.
+ * \param pkt Query packet.
  *
  * \retval KNOT_EOK on success.
  * \retval knot_error else.
  */
-int remote_parse(knot_packet_t* pkt, const uint8_t* buf, size_t buflen);
+int remote_parse(knot_pkt_t* pkt);
 
 /*!
  * \brief Execute command and prepare answer for client.
@@ -102,7 +103,7 @@ int remote_parse(knot_packet_t* pkt, const uint8_t* buf, size_t buflen);
  * \retval KNOT_EOK on success.
  * \retval knot_error else.
  */
-int remote_answer(int fd, server_t *s, knot_packet_t *pkt, uint8_t* rwire, size_t rlen);
+int remote_answer(int sock, server_t *s, knot_pkt_t *pkt);
 
 /*!
  * \brief Accept new client, receive command, process it and send response.
@@ -111,14 +112,14 @@ int remote_answer(int fd, server_t *s, knot_packet_t *pkt, uint8_t* rwire, size_
  *
  * \param s Server instance.
  * \param ctl_if Control interface.
- * \param r RC interface socket.
+ * \param sock RC interface socket.
  * \param buf Buffer for commands/responses.
  * \param buflen Maximum buffer size.
  *
  * \retval KNOT_EOK on success.
  * \retval knot_error else.
  */
-int remote_process(server_t *s, conf_iface_t *ctl_if, int r,
+int remote_process(server_t *s, conf_iface_t *ctl_if, int sock,
                    uint8_t* buf, size_t buflen);
 
 /* Functions for creating RC packets. */
@@ -134,18 +135,7 @@ int remote_process(server_t *s, conf_iface_t *ctl_if, int r,
  * \retval KNOT_EOK on success.
  * \retval knot_error else.
  */
-knot_packet_t* remote_query(const char *query, const knot_tsig_key_t *key);
-
-/*!
- * \brief Append extra data to RC command packet.
- *
- * \param qry RC packet.
- * \param data Extra data in form of a RR set.
- *
- * \retval KNOT_EOK on success.
- * \retval knot_error else.
- */
-int remote_query_append(knot_packet_t *qry, knot_rrset_t *data);
+knot_pkt_t* remote_query(const char *query, const knot_tsig_key_t *key);
 
 /*!
  * \brief Sign a RC command packet using TSIG key.
@@ -171,7 +161,7 @@ int remote_query_sign(uint8_t *wire, size_t *size, size_t maxlen,
  *
  * \return created RR set or NULL.
  */
-knot_rrset_t* remote_build_rr(const char *k, uint16_t t);
+int remote_build_rr(knot_rrset_t *rr, const char *k, uint16_t t);
 
 /*!
  * \brief Create a TXT rdata.
@@ -181,7 +171,6 @@ knot_rrset_t* remote_build_rr(const char *k, uint16_t t);
  */
 int remote_create_txt(knot_rrset_t *rr, const char *v, size_t v_len);
 
-
 /*!
  * \brief Create a CNAME rdata.
  * \param d Domain name as a string.
@@ -196,6 +185,4 @@ int remote_create_ns(knot_rrset_t *rr, const char *d);
  */
 int remote_print_txt(const knot_rrset_t *rrset, uint16_t i);
 
-#endif // _KNOTD_REMOTE_H_
-
 /*! @} */
