@@ -20,8 +20,8 @@
 #include <inttypes.h>
 
 #include "common/debug.h"
-#include "common/errcode.h"
-#include "common/mempattern.h"
+#include "common/mem.h"
+#include "libknot/errcode.h"
 #include "libknot/common.h"
 #include "libknot/dname.h"
 #include "libknot/consts.h"
@@ -106,6 +106,65 @@ static void set_zone_key_flags(const knot_key_params_t *params,
 	                 (params->time_delete == 0 || now < params->time_delete);
 }
 
+/*!
+ * \brief Check if there is a functional KSK and ZSK for each used algorithm.
+ */
+static int check_keys_validity(const knot_zone_keys_t *keys)
+{
+	assert(keys);
+
+	const int MAX_ALGORITHMS = KNOT_DNSSEC_ALG_ECDSAP384SHA384 + 1;
+	struct {
+		bool published;
+		bool ksk_enabled;
+		bool zsk_enabled;
+	} algorithms[MAX_ALGORITHMS];
+	memset(algorithms, 0, sizeof(algorithms));
+
+	/* Make a list of used algorithms */
+
+	const knot_zone_key_t *key = NULL;
+	WALK_LIST(key, keys->list) {
+		knot_dnssec_algorithm_t a = key->dnssec_key.algorithm;
+		assert(a < MAX_ALGORITHMS);
+
+		if (key->is_public) {
+			// public key creates a requirement for an algorithm
+			algorithms[a].published = true;
+
+			// need fully enabled ZSK and KSK for each algorithm
+			if (key->is_active) {
+				if (key->is_ksk) {
+					algorithms[a].ksk_enabled = true;
+				} else {
+					algorithms[a].zsk_enabled = true;
+				}
+			}
+		}
+	}
+
+	/* Validate enabled algorithms */
+
+	int enabled_count = 0;
+	for (int a = 0; a < MAX_ALGORITHMS; a++) {
+		if (!algorithms[a].published) {
+			continue;
+		}
+
+		if (!algorithms[a].ksk_enabled || !algorithms[a].zsk_enabled) {
+			return KNOT_DNSSEC_EMISSINGKEYTYPE;
+		}
+
+		enabled_count += 1;
+	}
+
+	if (enabled_count == 0) {
+		return KNOT_DNSSEC_ENOKEY;
+	}
+
+	return KNOT_EOK;
+}
+
 /*!
  * \brief Load zone keys from a key directory.
  *
@@ -123,14 +182,6 @@ int knot_load_zone_keys(const char *keydir_name, const knot_dname_t *zone_name,
 		return KNOT_DNSSEC_ENOKEYDIR;
 	}
 
-	char *zname = knot_dname_to_str(zone_name);
-	char *msgpref = sprintf_alloc("DNSSEC: Zone %s -", zname);
-	free(zname);
-	if (msgpref == NULL) {
-		closedir(keydir);
-		return KNOT_ENOMEM;
-	}
-
 	int result = KNOT_EOK;
 
 	struct dirent entry_buf = { 0 };
@@ -151,7 +202,6 @@ int knot_load_zone_keys(const char *keydir_name, const knot_dname_t *zone_name,
 		if (!path) {
 			ERR_ALLOC_FAILED;
 			closedir(keydir);
-			free(msgpref);
 			return KNOT_ENOMEM;
 		}
 
@@ -160,27 +210,24 @@ int knot_load_zone_keys(const char *keydir_name, const knot_dname_t *zone_name,
 		UNUSED(written);
 		assert(written == path_len);
 
-		dbg_dnssec_detail("loading key '%s'\n", path);
-
 		knot_key_params_t params = { 0 };
 		int ret = knot_load_key_params(path, &params);
 		free(path);
 
 		if (ret != KNOT_EOK) {
-			log_zone_warning("DNSSEC: Failed to load key %s: %s\n",
-			                  entry->d_name, knot_strerror(ret));
+			log_zone_warning(zone_name, "DNSSEC, failed to load "
+			                 "key, file '%s' (%s)",
+			                 entry->d_name, knot_strerror(ret));
 			knot_free_key_params(&params);
 			continue;
 		}
 
 		if (!knot_dname_is_equal(zone_name, params.name)) {
-			dbg_dnssec_detail("skipping key, different zone name\n");
 			knot_free_key_params(&params);
 			continue;
 		}
 
 		if (knot_get_key_type(&params) != KNOT_KEY_DNSSEC) {
-			dbg_dnssec_detail("skipping key, different purpose\n");
 			knot_free_key_params(&params);
 			continue;
 		}
@@ -193,14 +240,11 @@ int knot_load_zone_keys(const char *keydir_name, const knot_dname_t *zone_name,
 		memset(key, '\0', sizeof(*key));
 		set_zone_key_flags(&params, key);
 
-		dbg_dnssec_detail("next key event %" PRIu32 "\n", key->next_event);
-
 		if (!knot_dnssec_algorithm_is_zonesign(params.algorithm,
 		                                       nsec3_enabled)
 		) {
-			log_zone_notice("%s Ignoring key %d (%s): unknown "
-			                "algorithm or non-NSEC3 algorithm when"
-			                " NSEC3 is requested.\n", msgpref,
+			log_zone_notice(zone_name, "DNSSEC, ignoring key %5d, "
+			                "file '%s' (incompatible algorithm)",
 			                params.keytag, entry->d_name);
 			knot_free_key_params(&params);
 			free(key);
@@ -208,9 +252,9 @@ int knot_load_zone_keys(const char *keydir_name, const knot_dname_t *zone_name,
 		}
 
 		if (knot_get_zone_key(keys, params.keytag) != NULL) {
-			log_zone_notice("%s Ignoring key %d (%s): duplicate "
-			                "keytag.\n", msgpref, params.keytag,
-			                entry->d_name);
+			log_zone_notice(zone_name, "DNSSEC, ignoring key %5d, "
+					"file '%s' (duplicate keytag)",
+					params.keytag, entry->d_name);
 			knot_free_key_params(&params);
 			free(key);
 			continue;
@@ -218,16 +262,17 @@ int knot_load_zone_keys(const char *keydir_name, const knot_dname_t *zone_name,
 
 		ret = knot_dnssec_key_from_params(&params, &key->dnssec_key);
 		if (ret != KNOT_EOK) {
-			log_zone_error("%s Failed to process key %d (%s): %s\n",
-			               msgpref, params.keytag, entry->d_name,
+			log_zone_error(zone_name, "DNSSEC, failed to process "
+				       "key %5d, file '%s' (%s)",
+				       params.keytag, entry->d_name,
 			               knot_strerror(ret));
 			knot_free_key_params(&params);
 			free(key);
 			continue;
 		}
 
-		log_zone_info("%s - Loaded key %5d, file %s, %s, %s, %s\n",
-		              msgpref, params.keytag, entry->d_name,
+		log_zone_info(zone_name, "DNSSEC, loaded key %5d, file '%s', %s, %s, %s",
+		              params.keytag, entry->d_name,
 		              key->is_ksk ? "KSK" : "ZSK",
 		              key->is_active ? "active" : "inactive",
 		              key->is_public ? "public" : "not-public");
@@ -239,8 +284,8 @@ int knot_load_zone_keys(const char *keydir_name, const knot_dname_t *zone_name,
 
 	closedir(keydir);
 
-	if (result == KNOT_EOK && EMPTY_LIST(keys->list)) {
-		result = KNOT_DNSSEC_ENOKEY;
+	if (result == KNOT_EOK) {
+		result = check_keys_validity(keys);
 	}
 
 	if (result == KNOT_EOK) {
@@ -251,8 +296,6 @@ int knot_load_zone_keys(const char *keydir_name, const knot_dname_t *zone_name,
 		knot_free_zone_keys(keys);
 	}
 
-	free(msgpref);
-
 	return result;
 }
 
