@@ -121,7 +121,7 @@ static bool dname_cname_cannot_synth(const knot_rrset_t *rrset, const knot_dname
 static bool have_dnssec(struct query_data *qdata)
 {
 	return knot_pkt_has_dnssec(qdata->query) &&
-	       qdata->rcode_ext != KNOT_EDNS_RCODE_BADVERS &&
+	       qdata->rcode_ext != KNOT_RCODE_BADVERS &&
 	       zone_contents_is_signed(qdata->zone->contents);
 }
 
@@ -416,8 +416,8 @@ static int follow_cname(knot_pkt_t *pkt, uint16_t rrtype, struct query_data *qda
 	qdata->name = knot_cname_name(&cname_rr.rrs);
 
 #ifdef KNOT_NS_DEBUG
-	char *cname_str = knot_dname_to_str(cname_node->owner);
-	char *target_str = knot_dname_to_str(qdata->name);
+	char *cname_str = knot_dname_to_str_alloc(cname_node->owner);
+	char *target_str = knot_dname_to_str_alloc(qdata->name);
 	dbg_ns("%s: FOLLOW '%s' -> '%s'\n", __func__, cname_str, target_str);
 	free(cname_str);
 	free(target_str);
@@ -864,7 +864,7 @@ int internet_query_plan(struct query_plan *plan)
 /*! \brief Process answer to SOA query. */
 static int process_soa_answer(knot_pkt_t *pkt, struct answer_data *data)
 {
-	zone_t *zone  = data->param->zone;
+	zone_t *zone = data->param->zone;
 
 	/* Expect SOA in answer section. */
 	const knot_pktsection_t *answer = knot_pkt_section(pkt, KNOT_ANSWER);
@@ -884,6 +884,7 @@ static int process_soa_answer(knot_pkt_t *pkt, struct answer_data *data)
 	uint32_t their_serial =	knot_soa_serial(&answer->rr[0].rrs);
 	if (knot_serial_compare(our_serial, their_serial) >= 0) {
 		ANSWER_LOG(LOG_INFO, data, "refresh, outgoing", "zone is up-to-date");
+		zone_events_cancel(zone, ZONE_EVENT_EXPIRE);
 		return NS_PROC_DONE; /* Our zone is up to date. */
 	}
 
