@@ -24,6 +24,7 @@
 #include "libknot/rrset.h"
 #include "libknot/consts.h"
 #include "knot/zone/zonedb.h"
+#include "knot/zone/timers.h"
 #include "libknot/common.h"
 #include "libknot/packet/wire.h"
 #include "knot/updates/acl.h"
@@ -99,6 +100,10 @@ int notify_process_query(knot_pkt_t *pkt, struct query_data *qdata)
 	/* Incoming NOTIFY expires REFRESH timer and renews EXPIRE timer. */
 	zone_t *zone = (zone_t *)qdata->zone;
 	zone_events_schedule(zone, ZONE_EVENT_REFRESH, ZONE_EVENT_NOW);
+	int ret = zone_events_write_persistent(zone);
+	if (ret != KNOT_EOK) {
+		return NS_PROC_FAIL;
+	}
 
 	return NS_PROC_DONE;
 }
