@@ -21,13 +21,7 @@
  * \brief Core server functions.
  *
  * Contains the main high-level server structure (server_t) and interface
- * to functions taking care of proper initialization of the server and clean-up
- * when terminated.
- *
- * As of now, the server supports only one zone file and only in a special
- * format.
- *
- * \see zone-parser.h
+ * to functions taking care of initialization of the server and clean-up.
  *
  * \addtogroup server
  * @{
@@ -38,6 +32,7 @@
 #include "common-knot/evsched.h"
 #include "common-knot/lists.h"
 #include "common-knot/fdset.h"
+#include "common/namedb/namedb.h"
 #include "knot/server/dthreads.h"
 #include "knot/server/net.h"
 #include "knot/server/rrl.h"
@@ -59,11 +54,6 @@ typedef struct iohandler {
 	unsigned           *thread_id; /*!< Thread identifier. */
 } iohandler_t;
 
-/*! \brief Round-robin mechanism of switching.
-  */
-#define get_next_rr(current, count) \
-	(((current) + 1) % (count))
-
 /*! \brief Server state flags.
  */
 typedef enum {
@@ -104,7 +94,9 @@ typedef struct server_t {
 	/*! \brief Server state tracking. */
 	volatile unsigned state;
 
-	knot_zonedb_t *zone_db; /*!< Zone database. */
+	/*! \brief Zone database. */
+	knot_zonedb_t *zone_db;
+	knot_namedb_t *timers_db;
 
 	/*! \brief I/O handlers. */
 	unsigned tu_size;
