@@ -28,22 +28,24 @@
  * @{
  */
 
-#ifndef _KNOTD_UDPHANDLER_H_
-#define _KNOTD_UDPHANDLER_H_
+#pragma once
 
-#include "knot/server/socket.h"
+#include "knot/server/net.h"
 #include "knot/server/server.h"
 #include "knot/server/dthreads.h"
 
 /*!
- * \brief Create UDP handler context.
+ * \brief Send a UDP message.
+ *
+ * \param fd Associated socket.
+ * \param msg Buffer for a query wireformat.
+ * \param msglen Buffer maximum size.
+ * \param addr Destination address.
+ *
+ * \retval Number of sent data on success.
+ * \retval KNOT_ERROR on error.
  */
-void* udp_create_ctx(void);
-
-/*!
- * \brief Destroy UDP handler context.
- */
-void udp_free_ctx(void *ctx);
+int udp_send_msg(int fd, const uint8_t *msg, size_t msglen, struct sockaddr *addr);
 
 /*!
  * \brief UDP handler thread runnable.
@@ -64,6 +66,4 @@ int udp_master(dthread_t *thread);
  */
 int udp_master_destruct(dthread_t *thread);
 
-#endif
-
 /*! @} */
