@@ -28,7 +28,7 @@
 
 #pragma once
 
-#include "common/descriptor.h"
+#include "libknot/descriptor.h"
 #include "libknot/dname.h"
 #include "libknot/rrset.h"
 #include "libknot/rdataset.h"
@@ -86,18 +86,20 @@ enum node_flags {
  * \brief Creates and initializes new node structure.
  *
  * \param owner  Node's owner, will be duplicated.
+ * \param mm     Memory context to use.
  *
  * \return Newly created node or NULL if an error occured.
  */
-zone_node_t *node_new(const knot_dname_t *owner);
+zone_node_t *node_new(const knot_dname_t *owner, mm_ctx_t *mm);
 
 /*!
  * \brief Destroys allocated data within the node
  *        structure, but not the node itself.
  *
- * \param node Node that contains data to be destroyed.
+ * \param node  Node that contains data to be destroyed.
+ * \param mm    Memory context to use.
  */
-void node_free_rrsets(zone_node_t *node);
+void node_free_rrsets(zone_node_t *node, mm_ctx_t *mm);
 
 /*!
  * \brief Destroys the node structure.
@@ -105,18 +107,20 @@ void node_free_rrsets(zone_node_t *node);
  * Does not destroy the data within the node.
  * Also sets the given pointer to NULL.
  *
- * \param node Node to be destroyed.
+ * \param node  Node to be destroyed.
+ * \param mm    Memory context to use.
  */
-void node_free(zone_node_t **node);
+void node_free(zone_node_t **node, mm_ctx_t *mm);
 
 /*!
  * \brief Creates a shallow copy of node structure, RR data are shared.
  *
  * \param src  Source of the copy.
+ * \param mm   Memory context to use.
  *
  * \return Copied node if success, NULL otherwise.
  */
-zone_node_t *node_shallow_copy(const zone_node_t *src);
+zone_node_t *node_shallow_copy(const zone_node_t *src, mm_ctx_t *mm);
 
 /* ----------------------- Data addition/removal -----------------------------*/
 
@@ -129,7 +133,7 @@ zone_node_t *node_shallow_copy(const zone_node_t *src);
  *
  * \return KNOT_E*
  */
-int node_add_rrset(zone_node_t *node, const knot_rrset_t *rrset);
+int node_add_rrset(zone_node_t *node, const knot_rrset_t *rrset, mm_ctx_t *mm);
 
 /*!
  * \brief Removes data for given RR type from node.
@@ -194,6 +198,16 @@ bool node_rrtype_is_signed(const zone_node_t *node, uint16_t type);
  */
 bool node_rrtype_exists(const zone_node_t *node, uint16_t type);
 
+/*!
+ * \brief Checks whether node is empty. Node is empty when NULL or when no
+ *        RRSets are in it.
+ *
+ * \param node  Node to check in.
+ *
+ * \return True/False.
+ */
+bool node_empty(const zone_node_t *node);
+
 /* -------------------- Inline RRSet initializations ------------------------ */
 
 /*!
