@@ -24,22 +24,20 @@
  * @{
  */
 
-#ifndef _KNOTD_ZONEDUMP_H_
-#define _KNOTD_ZONEDUMP_H_
+#pragma once
 
-#include "libknot/zone/zone.h"
+#include "knot/zone/zone.h"
 
 /*!
  * \brief Dumps given zone to text file.
  *
  * \param zone Zone to be saved.
+ * \param from From where the zone originated.
  * \param file File to write to.
  *
  * \retval KNOT_EOK on success.
  * \retval < 0 if error.
  */
-int zone_dump_text(knot_zone_contents_t *zone, FILE *file);
-
-#endif // _KNOTD_ZONEDUMP_H_
+int zone_dump_text(zone_contents_t *zone, const struct sockaddr_storage *from, FILE *file);
 
 /*! @} */
