@@ -37,6 +37,7 @@
 #include "knot/zone/zonefile.h"
 #include "libknot/rdata.h"
 #include "knot/zone/zone-dump.h"
+#include "libknot/rrtype/naptr.h"
 
 #define ERROR(zone, fmt...) log_zone_error(zone, "zone loader, " fmt)
 #define WARNING(zone, fmt...) log_zone_warning(zone, "zone loader, " fmt)
@@ -63,7 +64,7 @@ static bool handle_err(zcreator_t *zc, const zone_node_t *node,
                        const knot_rrset_t *rr, int ret, bool master)
 {
 	const knot_dname_t *zname = zc->z->apex->owner;
-	char *rrname = rr ? knot_dname_to_str(rr->owner) : NULL;
+	char *rrname = rr ? knot_dname_to_str_alloc(rr->owner) : NULL;
 	if (ret == KNOT_EOUTOFZONE) {
 		WARNING(zname, "ignoring out-of-zone data, owner '%s'",
 		        rrname ? rrname : "unknown");
@@ -158,13 +159,12 @@ static void scanner_process(zs_scanner_t *scanner)
 		zc->ret = KNOT_ENOMEM;
 		return;
 	}
-	knot_dname_to_lower(owner);
 
 	knot_rrset_t rr;
 	knot_rrset_init(&rr, owner, scanner->r_type, scanner->r_class);
 	int ret = add_rdata_to_rr(&rr, scanner);
 	if (ret != KNOT_EOK) {
-		char *rr_name = knot_dname_to_str(rr.owner);
+		char *rr_name = knot_dname_to_str_alloc(rr.owner);
 		const knot_dname_t *zname = zc->z->apex->owner;
 		ERROR(zname, "failed to add RDATA, file '%s', line %"PRIu64", owner '%s'",
 		      scanner->file.name, scanner->line_counter, rr_name);
@@ -174,6 +174,14 @@ static void scanner_process(zs_scanner_t *scanner)
 		return;
 	}
 
+	/* Convert RDATA dnames to lowercase before adding to zone. */
+	ret = knot_rrset_rr_to_canonical(&rr);
+	if (ret != KNOT_EOK) {
+		knot_dname_free(&owner, NULL);
+		zc->ret = ret;
+		return;
+	}
+
 	zc->ret = zcreator_step(zc, &rr);
 	knot_dname_free(&owner, NULL);
 	knot_rdataset_clear(&rr.rrs, NULL);
@@ -184,7 +192,7 @@ static zone_contents_t *create_zone_from_name(const char *origin)
 	if (origin == NULL) {
 		return NULL;
 	}
-	knot_dname_t *owner = knot_dname_from_str(origin);
+	knot_dname_t *owner = knot_dname_from_str_alloc(origin);
 	if (owner == NULL) {
 		return NULL;
 	}
