@@ -29,6 +29,7 @@
 #include "libknot/packet/rrset-wire.h"
 #include "libknot/packet/wire.h"
 #include "libknot/rrset.h"
+#include "libknot/rrtype/naptr.h"
 
 /*!
  * \brief Get maximal size of a domain name in a wire with given capacity.
@@ -114,26 +115,14 @@ static int write_rdata_naptr_header(const uint8_t **src, size_t *src_avail,
 	assert(dst && *dst);
 	assert(dst_avail);
 
-	size_t size = 0;
+	int ret = knot_naptr_header_size(*src, *src + *src_avail);
 
-	/* Fixed fields size (order, preference) */
-
-	size += 2 * sizeof(uint16_t);
-
-	/* Variable fields size (flags, services, regexp) */
-
-	for (int i = 0; i < 3; i++) {
-		const uint8_t *len_ptr = *src + size;
-		if (len_ptr >= *src + *src_avail) {
-			return KNOT_EMALF;
-		}
-
-		size += 1 + *len_ptr;
+	if (ret < 0) {
+		return ret;
 	}
 
 	/* Copy the data */
-
-	return write_rdata_fixed(src, src_avail, dst, dst_avail, size);
+	return write_rdata_fixed(src, src_avail, dst, dst_avail, ret);
 }
 
 /*!
@@ -142,8 +131,7 @@ static int write_rdata_naptr_header(const uint8_t **src, size_t *src_avail,
 struct dname_config {
 	int (*write_cb)(const uint8_t **src, size_t *src_avail,
 	                uint8_t **dst, size_t *dst_avail,
-	                int dname_type, struct dname_config *dname_cfg,
-	                knot_rrset_wire_flags_t flags);
+	                int dname_type, struct dname_config *dname_cfg);
 	knot_compr_t *compr;
 	uint16_t hint;
 	const uint8_t *pkt_wire;
@@ -156,15 +144,14 @@ typedef struct dname_config dname_config_t;
  */
 static int write_rdata_block(const uint8_t **src, size_t *src_avail,
                              uint8_t **dst, size_t *dst_avail,
-                             int type, dname_config_t *dname_cfg,
-                             knot_rrset_wire_flags_t flags)
+                             int type, dname_config_t *dname_cfg)
 {
 	switch (type) {
 	case KNOT_RDATA_WF_COMPRESSIBLE_DNAME:
 	case KNOT_RDATA_WF_DECOMPRESSIBLE_DNAME:
 	case KNOT_RDATA_WF_FIXED_DNAME:
 		return dname_cfg->write_cb(src, src_avail, dst, dst_avail,
-		                           type, dname_cfg, flags);
+		                           type, dname_cfg);
 	case KNOT_RDATA_WF_NAPTR_HEADER:
 		return write_rdata_naptr_header(src, src_avail, dst, dst_avail);
 	case KNOT_RDATA_WF_REMAINDER:
@@ -182,12 +169,12 @@ static int write_rdata_block(const uint8_t **src, size_t *src_avail,
 static int rdata_traverse(const uint8_t **src, size_t *src_avail,
                           uint8_t **dst, size_t *dst_avail,
                           const rdata_descriptor_t *desc,
-                          dname_config_t *dname_cfg, knot_rrset_wire_flags_t flags)
+                          dname_config_t *dname_cfg)
 {
 	for (int i = 0; desc->block_types[i] != KNOT_RDATA_WF_END; i++) {
 		int block_type = desc->block_types[i];
 		int ret = write_rdata_block(src, src_avail, dst, dst_avail,
-		                            block_type, dname_cfg, flags);
+		                            block_type, dname_cfg);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
@@ -202,7 +189,7 @@ static int rdata_traverse(const uint8_t **src, size_t *src_avail,
  * \brief Write RR owner to wire.
  */
 static int write_owner(const knot_rrset_t *rrset, uint8_t **dst, size_t *dst_avail,
-                       knot_compr_t *compr, knot_rrset_wire_flags_t flags)
+                       knot_compr_t *compr)
 {
 	assert(rrset);
 	assert(dst && *dst);
@@ -234,11 +221,6 @@ static int write_owner(const knot_rrset_t *rrset, uint8_t **dst, size_t *dst_ava
 			return written;
 		}
 
-		if (flags & KNOT_RRSET_WIRE_CANONICAL) {
-			assert(compr == NULL);
-			knot_dname_to_lower(*dst);
-		}
-
 		compr_set_ptr(compr, COMPR_HINT_OWNER, *dst, written);
 		owner_size = written;
 	}
@@ -299,8 +281,7 @@ static int write_fixed_header(const knot_rrset_t *rrset, uint16_t rrset_index,
  */
 static int compress_rdata_dname(const uint8_t **src, size_t *src_avail,
                                 uint8_t **dst, size_t *dst_avail,
-                                int dname_type, dname_config_t *dname_cfg,
-                                knot_rrset_wire_flags_t flags)
+                                int dname_type, dname_config_t *dname_cfg)
 {
 	assert(src && *src);
 	assert(src_avail);
@@ -327,13 +308,6 @@ static int compress_rdata_dname(const uint8_t **src, size_t *src_avail,
 		return written;
 	}
 
-	/* Post-processing */
-
-	if (flags & KNOT_RRSET_WIRE_CANONICAL) {
-		assert(dname_cfg->compr == NULL);
-		knot_dname_to_lower(*dst);
-	}
-
 	/* Update compression hints */
 
 	if (compr_get_ptr(dname_cfg->compr, dname_cfg->hint) == 0) {
@@ -355,8 +329,7 @@ static int compress_rdata_dname(const uint8_t **src, size_t *src_avail,
  * \brief Write RDLENGTH and RDATA fields of a RR in a wire.
  */
 static int write_rdata(const knot_rrset_t *rrset, uint16_t rrset_index,
-                       uint8_t **dst, size_t *dst_avail,
-                       knot_compr_t *compr, knot_rrset_wire_flags_t flags)
+                       uint8_t **dst, size_t *dst_avail, knot_compr_t *compr)
 {
 	assert(rrset);
 	assert(rrset_index < rrset->rrs.rr_count);
@@ -391,7 +364,7 @@ static int write_rdata(const knot_rrset_t *rrset, uint16_t rrset_index,
 		const rdata_descriptor_t *desc =
 			knot_get_rdata_descriptor(rrset->type);
 		int ret = rdata_traverse(&src, &src_avail, dst, dst_avail,
-		                         desc, &dname_cfg, flags);
+		                         desc, &dname_cfg);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
@@ -414,12 +387,11 @@ static int write_rdata(const knot_rrset_t *rrset, uint16_t rrset_index,
  * \brief Write one RR from a RR Set to wire.
  */
 static int write_rr(const knot_rrset_t *rrset, uint16_t rrset_index,
-                    uint8_t **dst, size_t *dst_avail, knot_compr_t *compr,
-                    knot_rrset_wire_flags_t flags)
+                    uint8_t **dst, size_t *dst_avail, knot_compr_t *compr)
 {
 	int ret;
 
-	ret = write_owner(rrset, dst, dst_avail, compr, flags);
+	ret = write_owner(rrset, dst, dst_avail, compr);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -429,28 +401,24 @@ static int write_rr(const knot_rrset_t *rrset, uint16_t rrset_index,
 		return ret;
 	}
 
-	return write_rdata(rrset, rrset_index, dst, dst_avail, compr, flags);
+	return write_rdata(rrset, rrset_index, dst, dst_avail, compr);
 }
 
 /*!
  * \brief Write RR Set content to a wire.
  */
 int knot_rrset_to_wire(const knot_rrset_t *rrset, uint8_t *wire, uint16_t max_size,
-                       knot_compr_t *compr, knot_rrset_wire_flags_t flags)
+                       knot_compr_t *compr)
 {
 	if (!rrset || !wire) {
 		return KNOT_EINVAL;
 	}
 
-	if (flags & KNOT_RRSET_WIRE_CANONICAL) {
-		compr = NULL;
-	}
-
 	uint8_t *write = wire;
 	size_t capacity = max_size;
 
 	for (uint16_t i = 0; i < rrset->rrs.rr_count; i++) {
-		int ret = write_rr(rrset, i, &write, &capacity, compr, flags);
+		int ret = write_rr(rrset, i, &write, &capacity, compr);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
@@ -483,7 +451,6 @@ static int parse_header(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 	if (owner == NULL) {
 		return KNOT_EMALF;
 	}
-	knot_dname_to_lower(owner);
 
 	if (pkt_size - *pos < RR_HEADER_SIZE) {
 		knot_dname_free(&owner, mm);
@@ -514,22 +481,23 @@ static int parse_header(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
  */
 static int decompress_rdata_dname(const uint8_t **src, size_t *src_avail,
                                   uint8_t **dst, size_t *dst_avail,
-                                  int dname_type, dname_config_t *dname_cfg,
-                                  knot_rrset_wire_flags_t flags)
+                                  int dname_type, dname_config_t *dname_cfg)
 {
 	assert(src && *src);
 	assert(src_avail);
 	assert(dst && *dst);
 	assert(dst_avail);
 	assert(dname_cfg);
-	UNUSED(flags);
+	UNUSED(dname_type);
 
-	int compr_size = knot_dname_wire_check(*src, *src + *src_avail, dname_cfg->pkt_wire);
+	int compr_size = knot_dname_wire_check(*src, *src + *src_avail,
+	                                       dname_cfg->pkt_wire);
 	if (compr_size <= 0) {
 		return compr_size;
 	}
 	
-	int decompr_size = knot_dname_unpack(*dst, *src, *dst_avail, dname_cfg->pkt_wire);
+	int decompr_size = knot_dname_unpack(*dst, *src, *dst_avail,
+	                                     dname_cfg->pkt_wire);
 	if (decompr_size <= 0) {
 		return decompr_size;
 	}
@@ -596,8 +564,7 @@ static int parse_rdata(const uint8_t *pkt_wire, size_t *pos, size_t pkt_size,
 		.pkt_wire = pkt_wire
 	};
 
-	int ret = rdata_traverse(&src, &src_avail, &dst, &dst_avail,
-	                         desc, &dname_cfg, KNOT_RRSET_WIRE_NONE);
+	int ret = rdata_traverse(&src, &src_avail, &dst, &dst_avail, desc, &dname_cfg);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -642,7 +609,15 @@ int knot_rrset_rr_from_wire(const uint8_t *pkt_wire, size_t *pos,
 	ret = parse_rdata(pkt_wire, pos, pkt_size, mm, ttl, rdlen, rrset);
 	if (ret != KNOT_EOK) {
 		knot_rrset_clear(rrset, mm);
+		return ret;
 	}
 
-	return ret;
+	/* Convert RR to canonical format. */
+	ret = knot_rrset_rr_to_canonical(rrset);
+
+	if (ret != KNOT_EOK) {
+		knot_rrset_clear(rrset, mm);
+	}
+
+	return KNOT_EOK;
 }
