@@ -25,6 +25,7 @@
 #include "libknot/mempattern.h"
 #include "libknot/descriptor.h"
 #include "libknot/dname.h"
+#include "libknot/rrtype/naptr.h"
 
 knot_rrset_t *knot_rrset_new(const knot_dname_t *owner, uint16_t type,
                              uint16_t rclass, mm_ctx_t *mm)
@@ -158,3 +159,63 @@ uint32_t knot_rrset_ttl(const knot_rrset_t *rrset)
 	return knot_rdata_ttl(knot_rdataset_at(&(rrset->rrs), 0));
 }
 
+int knot_rrset_rr_to_canonical(knot_rrset_t *rrset)
+{
+	if (rrset == NULL || rrset->rrs.rr_count != 1) {
+		return KNOT_EINVAL;
+	}
+
+	/* Convert owner for all RRSets. */
+	int ret = knot_dname_to_lower(rrset->owner);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	/* Convert DNAMEs in RDATA only for RFC4034 types. */
+	if (!knot_rrtype_should_be_lowercased(rrset->type)) {
+		return KNOT_EOK;
+	}
+
+	const rdata_descriptor_t *desc = knot_get_rdata_descriptor(rrset->type);
+	if (desc->type_name == NULL) {
+		desc = knot_get_obsolete_rdata_descriptor(rrset->type);
+	}
+
+	knot_rdata_t *rdata = knot_rdataset_at(&rrset->rrs, 0);
+	assert(rdata);
+	uint16_t rdlen = knot_rdata_rdlen(rdata);
+	uint8_t *pos = knot_rdata_data(rdata);
+
+	/* No RDATA */
+	if (rdlen == 0) {
+		return KNOT_EOK;
+	}
+
+	/* Otherwise, whole and not malformed RDATA are expected. */
+	for (int i = 0; desc->block_types[i] != KNOT_RDATA_WF_END; ++i) {
+		int type = desc->block_types[i];
+		switch (type) {
+		case KNOT_RDATA_WF_COMPRESSIBLE_DNAME:
+		case KNOT_RDATA_WF_DECOMPRESSIBLE_DNAME:
+		case KNOT_RDATA_WF_FIXED_DNAME:
+			ret = knot_dname_to_lower(pos);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+			pos += knot_dname_size(pos);
+			break;
+		case KNOT_RDATA_WF_NAPTR_HEADER:
+			pos += knot_naptr_header_size(pos, rdata + rdlen);
+			break;
+		case KNOT_RDATA_WF_REMAINDER:
+			break;
+		default:
+			/* Fixed size block */
+			assert(type > 0);
+			pos += type;
+		}
+	}
+
+	return KNOT_EOK;
+}
+
