@@ -30,7 +30,7 @@
 #include <netdb.h>			// addrinfo
 #include <sys/socket.h>			// sockaddr_storage (BSD)
 
-#include "common/lists.h"		// node
+#include "common-knot/lists.h"		// node
 #include "utils/common/params.h"	// params_t
 
 /*! \brief Structure containing server information. */
