@@ -26,7 +26,7 @@
 
 #pragma once
 
-#include "common/lists.h"		// list
+#include "common-knot/lists.h"		// list
 #include "utils/common/netio.h"		// server_t
 
 srv_info_t* parse_nameserver(const char *str, const char *def_port);
