@@ -27,139 +27,136 @@ typedef struct {
 #define ERR_ITEM(code, text) { code, text, #code }
 
 const err_table_t err_msgs[] = {
-	ERR_ITEM( ZS_OK, "OK" ),
+	ERR_ITEM( ZS_OK, "ok" ),
 
-	/* Zone file loader errors. */
-	ERR_ITEM( ZS_LOADER_FSTAT,
-	          "Fstat error." ),
-	ERR_ITEM( ZS_LOADER_DIRECTORY,
-	          "Zone file is a directory." ),
-	ERR_ITEM( ZS_LOADER_EMPTY,
-	          "Empty zone file." ),
-	ERR_ITEM( ZS_LOADER_MMAP,
-	          "Mmap error." ),
-	ERR_ITEM( ZS_LOADER_MUNMAP,
-	          "Munmap error." ),
-	ERR_ITEM( ZS_LOADER_SCANNER,
-	          "Zone processing error." ),
-
-	/* Zone scanner errors. */
+	ERR_ITEM( ZS_SYSTEM,
+	          "system error" ),
+	ERR_ITEM( ZS_FILE_OPEN,
+	          "file open error" ),
+	ERR_ITEM( ZS_FILE_PATH,
+	          "file path error" ),
+	ERR_ITEM( ZS_FILE_FSTAT,
+	          "fstat error" ),
+	ERR_ITEM( ZS_FILE_DIR,
+	          "not zone file but directory" ),
+	ERR_ITEM( ZS_FILE_MMAP,
+	          "file mmap error" ),
 	ERR_ITEM( ZS_DOS_NEWLINE,
-	          "Unsupported CRLF newline. Please, remove CR bytes." ),
+	          "unsupported CRLF newline, remove CR bytes" ),
 	ERR_ITEM( ZS_UNCOVERED_STATE,
-	          "General scanner error." ),
+	          "general scanner error" ),
 	ERR_ITEM( ZS_UNCLOSED_MULTILINE,
-	          "Unclosed last multiline block." ),
+	          "unclosed last multiline block" ),
 	ERR_ITEM( ZS_LEFT_PARENTHESIS,
-	          "Too many left parentheses." ),
+	          "too many left parentheses" ),
 	ERR_ITEM( ZS_RIGHT_PARENTHESIS,
-	          "Too many right parentheses." ),
+	          "too many right parentheses" ),
 	ERR_ITEM( ZS_UNSUPPORTED_TYPE,
-	          "Unsupported record type." ),
+	          "unsupported record type" ),
 	ERR_ITEM( ZS_BAD_PREVIOUS_OWNER,
-	          "Previous owner is invalid." ),
+	          "previous owner is invalid" ),
 	ERR_ITEM( ZS_BAD_DNAME_CHAR,
-	          "Invalid domain name character." ),
+	          "invalid domain name character" ),
 	ERR_ITEM( ZS_BAD_OWNER,
-	          "Owner is invalid." ),
+	          "owner is invalid" ),
 	ERR_ITEM( ZS_LABEL_OVERFLOW,
-	          "Maximal domain name label length has exceeded." ),
+	          "maximal domain name label length has exceeded" ),
 	ERR_ITEM( ZS_DNAME_OVERFLOW,
-	          "Maximal domain name length has exceeded." ),
+	          "maximal domain name length has exceeded" ),
 	ERR_ITEM( ZS_BAD_NUMBER,
-	          "Invalid number." ),
+	          "invalid number" ),
 	ERR_ITEM( ZS_NUMBER64_OVERFLOW,
-	          "Number is too big." ),
+	          "number is too big" ),
 	ERR_ITEM( ZS_NUMBER32_OVERFLOW,
-	          "Number is bigger than 32 bits." ),
+	          "number is bigger than 32 bits" ),
 	ERR_ITEM( ZS_NUMBER16_OVERFLOW,
-	          "Number is bigger than 16 bits." ),
+	          "number is bigger than 16 bits" ),
 	ERR_ITEM( ZS_NUMBER8_OVERFLOW,
-	          "Number is bigger than 8 bits." ),
+	          "number is bigger than 8 bits" ),
 	ERR_ITEM( ZS_FLOAT_OVERFLOW,
-	          "Float number overflow." ),
+	          "float number overflow" ),
 	ERR_ITEM( ZS_RDATA_OVERFLOW,
-	          "Maximal record data length has exceeded." ),
+	          "maximal record data length has exceeded" ),
 	ERR_ITEM( ZS_ITEM_OVERFLOW,
-	          "Maximal item length has exceeded." ),
+	          "maximal item length has exceeded" ),
 	ERR_ITEM( ZS_BAD_ADDRESS_CHAR,
-	          "Invalid address character." ),
+	          "invalid address character" ),
 	ERR_ITEM( ZS_BAD_IPV4,
-	          "Invalid IPv4 address." ),
+	          "invalid IPv4 address" ),
 	ERR_ITEM( ZS_BAD_IPV6,
-	          "Invalid IPv6 address." ),
+	          "invalid IPv6 address" ),
 	ERR_ITEM( ZS_BAD_GATEWAY,
-	          "Invalid gateway." ),
+	          "invalid gateway" ),
 	ERR_ITEM( ZS_BAD_GATEWAY_KEY,
-	          "Invalid gateway key." ),
+	          "invalid gateway key" ),
 	ERR_ITEM( ZS_BAD_APL,
-	          "Invalid address prefix list." ),
+	          "invalid address prefix list" ),
 	ERR_ITEM( ZS_BAD_RDATA,
-	          "Invalid record data." ),
+	          "invalid record data" ),
 	ERR_ITEM( ZS_BAD_HEX_RDATA,
-	          "Invalid record data in hex format." ),
+	          "invalid record data in hex format" ),
 	ERR_ITEM( ZS_BAD_HEX_CHAR,
-	          "Invalid hexadecimal character." ),
+	          "invalid hexadecimal character" ),
 	ERR_ITEM( ZS_BAD_BASE64_CHAR,
-	          "Invalid Base64 character." ),
+	          "invalid Base64 character" ),
 	ERR_ITEM( ZS_BAD_BASE32HEX_CHAR,
-	          "Invalid Base32hex character." ),
+	          "invalid Base32hex character" ),
 	ERR_ITEM( ZS_BAD_REST,
-	          "Unexpected data." ),
+	          "unexpected data" ),
 	ERR_ITEM( ZS_BAD_TIMESTAMP_CHAR,
-	          "Invalid timestamp character." ),
+	          "invalid timestamp character" ),
 	ERR_ITEM( ZS_BAD_TIMESTAMP_LENGTH,
-	          "Invalid timestamp length." ),
+	          "invalid timestamp length" ),
 	ERR_ITEM( ZS_BAD_TIMESTAMP,
-	          "Invalid timestamp." ),
+	          "invalid timestamp" ),
 	ERR_ITEM( ZS_BAD_DATE,
-	          "Invalid date." ),
+	          "invalid date" ),
 	ERR_ITEM( ZS_BAD_TIME,
-	          "Invalid time." ),
+	          "invalid time" ),
 	ERR_ITEM( ZS_BAD_TIME_UNIT,
-	          "Invalid time unit." ),
+	          "invalid time unit" ),
 	ERR_ITEM( ZS_BAD_BITMAP,
-	          "Invalid bitmap." ),
+	          "invalid bitmap" ),
 	ERR_ITEM( ZS_TEXT_OVERFLOW,
-	          "Text is too long." ),
+	          "text is too long" ),
 	ERR_ITEM( ZS_BAD_TEXT_CHAR,
-	          "Invalid text character." ),
+	          "invalid text character" ),
 	ERR_ITEM( ZS_BAD_TEXT,
-	          "Invalid text string." ),
+	          "invalid text string" ),
 	ERR_ITEM( ZS_BAD_DIRECTIVE,
-	          "Invalid directive." ),
+	          "invalid directive" ),
 	ERR_ITEM( ZS_BAD_TTL,
-	          "Invalid zone TTL." ),
+	          "invalid zone TTL" ),
 	ERR_ITEM( ZS_BAD_ORIGIN,
-	          "Invalid FQDN zone origin." ),
+	          "invalid FQDN zone origin" ),
 	ERR_ITEM( ZS_BAD_INCLUDE_FILENAME,
-	          "Invalid filename in include directive." ),
+	          "invalid filename in include directive" ),
 	ERR_ITEM( ZS_BAD_INCLUDE_ORIGIN,
-	          "Invalid origin in include directive." ),
+	          "invalid origin in include directive" ),
 	ERR_ITEM( ZS_UNPROCESSED_INCLUDE,
-	          "Include file processing error." ),
+	          "include file processing error" ),
 	ERR_ITEM( ZS_UNOPENED_INCLUDE,
-	          "Include file opening error." ),
+	          "include file opening error" ),
 	ERR_ITEM( ZS_BAD_RDATA_LENGTH,
-	          "The rdata length statement is incorrect." ),
+	          "the rdata length statement is incorrect" ),
 	ERR_ITEM( ZS_CANNOT_TEXT_DATA,
-	          "Unable to process text form for this type." ),
+	          "unable to process text form for this type" ),
 	ERR_ITEM( ZS_BAD_LOC_DATA,
-	          "Invalid zone location data." ),
+	          "invalid zone location data" ),
 	ERR_ITEM( ZS_UNKNOWN_BLOCK,
-	          "Unknown rdata block." ),
+	          "unknown rdata block" ),
 	ERR_ITEM( ZS_BAD_ALGORITHM,
-	          "Invalid algorithm." ),
+	          "invalid algorithm" ),
 	ERR_ITEM( ZS_BAD_CERT_TYPE,
-	          "Invalid certificate type." ),
+	          "invalid certificate type" ),
 	ERR_ITEM( ZS_BAD_EUI_LENGTH,
-	          "Invalid EUI length." ),
+	          "invalid EUI length" ),
 	ERR_ITEM( ZS_BAD_L64_LENGTH,
-	          "Invalid 64-bit locator." ),
+	          "invalid 64-bit locator" ),
 	ERR_ITEM( ZS_BAD_CHAR_COLON,
-	          "Missing colon character." ),
+	          "missing colon character" ),
 	ERR_ITEM( ZS_BAD_CHAR_DASH,
-	          "Missing dash character." ),
+	          "missing dash character" ),
 
 	ERR_ITEM( 0, NULL ) // Terminator
 };
