@@ -27,13 +27,12 @@
 enum err_codes {
 	ZS_OK = 0,
 
-	ZS_LOADER_FSTAT = -1000,
-	ZS_LOADER_DIRECTORY,
-	ZS_LOADER_EMPTY,
-	ZS_LOADER_MMAP,
-	ZS_LOADER_MUNMAP,
-	ZS_LOADER_SCANNER,
-
+	ZS_SYSTEM = -1000,
+	ZS_FILE_OPEN,
+	ZS_FILE_PATH,
+	ZS_FILE_FSTAT,
+	ZS_FILE_DIR,
+	ZS_FILE_MMAP,
 	ZS_DOS_NEWLINE,
 	ZS_UNCOVERED_STATE,
 	ZS_UNCLOSED_MULTILINE,
