@@ -18,8 +18,8 @@
 #include <assert.h>
 #include <tap/basic.h>
 
-#include "common/hhash.h"
-#include "common/mempattern.h"
+#include "common-knot/hhash.h"
+#include "libknot/mempattern.h"
 #include "common/mempool.h"
 #include "libknot/common.h"
 
