@@ -102,7 +102,7 @@ int main(int argc, char *argv[])
 		return KNOT_ERROR; /* Fatal */
 	}
 
-	knot_dname_t *qname = knot_dname_from_str("beef.");
+	knot_dname_t *qname = knot_dname_from_str_alloc("beef.");
 	int ret = knot_pkt_put_question(query, qname, KNOT_CLASS_IN, KNOT_RRTYPE_A);
 	knot_dname_free(&qname, NULL);
 	if (ret != KNOT_EOK) {
