@@ -18,7 +18,7 @@
 
 #include "common-knot/evsched.h"
 #include "knot/worker/pool.h"
-#include "knot/zone/events.h"
+#include "knot/zone/events/events.h"
 #include "knot/zone/zone.h"
 
 static void test_scheduling(zone_t *zone)
@@ -86,7 +86,7 @@ int main(void)
 	r = zone_events_init(&zone);
 	ok(r == KNOT_EOK, "zone events init");
 
-	r = zone_events_setup(&zone, pool, &sched);
+	r = zone_events_setup(&zone, pool, &sched, NULL);
 	ok(r == KNOT_EOK, "zone events setup");
 
 	test_scheduling(&zone);
