@@ -24,10 +24,10 @@ static zone_node_t NODE[NCOUNT];
 static knot_dname_t* ORDER[NCOUNT];
 static void ztree_init_data()
 {
-	NAME[0] = knot_dname_from_str(".");
-	NAME[1] = knot_dname_from_str("master.ac.");
-	NAME[2] = knot_dname_from_str("ac.");
-	NAME[3] = knot_dname_from_str("ns.");
+	NAME[0] = knot_dname_from_str_alloc(".");
+	NAME[1] = knot_dname_from_str_alloc("master.ac.");
+	NAME[2] = knot_dname_from_str_alloc("ac.");
+	NAME[3] = knot_dname_from_str_alloc("ns.");
 
 	knot_dname_t *order[NCOUNT] = {
 	        NAME[0], NAME[2], NAME[1], NAME[3]
@@ -55,8 +55,8 @@ static int ztree_iter_data(zone_node_t **node, void *data)
 	int result = KNOT_EOK;
 	if (owner != ORDER[*i]) {
 		result = KNOT_ERROR;
-		char *exp_s = knot_dname_to_str(ORDER[*i]);
-		char *owner_s = knot_dname_to_str(owner);
+		char *exp_s = knot_dname_to_str_alloc(ORDER[*i]);
+		char *owner_s = knot_dname_to_str_alloc(owner);
 		diag("ztree: at index: %u expected '%s' got '%s'\n", *i, exp_s, owner_s);
 		free(exp_s);
 		free(owner_s);
@@ -103,7 +103,7 @@ int main(int argc, char *argv[])
 	/* 4. ordered lookup */
 	node = NULL;
 	const zone_node_t *prev = NULL;
-	knot_dname_t *tmp_dn = knot_dname_from_str("z.ac.");
+	knot_dname_t *tmp_dn = knot_dname_from_str_alloc("z.ac.");
 	zone_tree_find_less_or_equal(t, tmp_dn, &node, &prev);
 	knot_dname_free(&tmp_dn, NULL);
 	ok(prev == NODE + 1, "ztree: ordered lookup");
